#!/usr/bin/env python
# -*- coding: utf-8 -*-

###############################################################################
# The MIT License
#
# Copyright (c) 2016 Grigory Chernyshev
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.
#
###############################################################################

from yagocd.resources import BaseManager


class ConfigurationManager(BaseManager):
    """
    The configuration API allows users with administration role
    to view and manage configuration.
    """

    def modifications(self):
        """
        Lists the config repository modifications.

        :return: An array of repository modifications.
        """
        response = self._session.get(
            path='{base_api}/config/revisions'.format(base_api=self.base_api),
            headers={'Accept': 'application/json'},
        )

        return response.json()

    def diff(self, start, end):
        """
        Gets the diff between two config repository modifications.

        :param start: starting SHA commit.
        :param end: ending SHA commit.
        :return: the diff between two config repo modifications.
        """
        response = self._session.get(
            path='{base_api}/config/diff/{start}/{end}'.format(
                base_api=self.base_api,
                start=start,
                end=end
            ),
            headers={'Accept': 'text/plain'},
        )

        return response.text

    def config(self, md5=None):
        """
        Gets the current configuration file.

        :param md5: md5 sum of config to get. If not given, current will be returned.
        :return: the contents of the configuration file.
        """
        response = self._session.get(
            path='{base_api}/admin/config/{version}'.format(
                base_api=self.base_api,
                version='current.xml' if md5 is None else (md5 + '.xml'),
            ),
            headers={'Accept': 'application/xml'},
        )

        return response.text


if __name__ == '__main__':
    pass



#!/usr/bin/env python
# -*- coding: utf-8 -*-

###############################################################################
# The MIT License
#
# Copyright (c) 2016 Grigory Chernyshev
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.
#
###############################################################################

import copy
from urlparse import urljoin

import requests


class Session(object):
    """
    Class for working with sessions.
    Instance of this class is created only once in the initialization of ``yagocd.client.Client`` and then passed
    as a parameter for all managers.
    """
    def __init__(self, auth, options):
        self._auth = auth
        self._options = options
        self._session = requests.Session()

    @staticmethod
    def urljoin(*args):
        """
        Joins given arguments into a url. Trailing but not leading slashes are
        stripped for each argument.
        """
        return "/".join(map(lambda x: str(x).rstrip('/').rstrip('/'), args)).rstrip('/')

    @property
    def server_url(self):
        """
        Property for getting server url.
        :return: server url for this instance.
        """
        return self._options['server'].rstrip('/')

    def request(self, method, path, params=None, data=None, headers=None, files=None):
        # this should work even if path is absolute (e.g. for files)
        url = urljoin(self._options['server'], path)

        merged_headers = copy.deepcopy(self._options['headers'])
        merged_headers.update(headers or {})

        response = self._session.request(
            method=method,
            url=url,
            params=params,
            data=data,
            headers=merged_headers,
            files=files,
            auth=self._auth,
            verify=self._options['verify']
        )
        # raise exception if we got 4xx/5xx response
        response.raise_for_status()

        return response

    def get(self, path, params=None, headers=None):
        return self.request(method='get', path=path, params=params, headers=headers)

    def post(self, path, params=None, data=None, headers=None, files=None):
        return self.request(method='post', path=path, params=params, data=data, headers=headers, files=files)

    def put(self, path, data=None, headers=None, files=None):
        return self.request(method='put', path=path, data=data, headers=headers, files=files)

    def patch(self, path, data=None, headers=None):
        return self.request(method='patch', path=path, data=data, headers=headers)

    def delete(self, path, data=None, headers=None):
        return self.request(method='delete', path=path, data=data, headers=headers)

    def base_api(self, context_path=None, api_path=None):
        return self.urljoin(
            context_path if context_path is not None else self._options['context_path'],
            api_path if api_path is not None else self._options['api_path']
        )


if __name__ == '__main__':
    pass



#!/usr/bin/env python
# -*- coding: utf-8 -*-

###############################################################################
# The MIT License
#
# Copyright (c) 2016 Grigory Chernyshev
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.
#
###############################################################################

from yagocd.resources import BaseManager, Base
from yagocd.resources.artifact import ArtifactManager
from yagocd.resources.property import PropertyManager


class JobManager(BaseManager):
    """
    The jobs API allows users to view job information.
    """

    def scheduled(self):
        """
        Lists all the current job instances which are scheduled but not yet assigned to any agent.

        :return: an array of scheduled job instances in XML format.
        """
        response = self._session.get(
            path='{base_api}/jobs/scheduled.xml'.format(base_api=self.base_api),
            headers={'Accept': 'application/xml'},
        )

        return response.text

    def history(self, pipeline_name, stage_name, job_name, offset=0):
        """
        The job history allows users to list job instances of specified job.
        Supports pagination using offset which tells the API how many instances to skip.

        :return: an array of jobs instances.
        """
        response = self._session.get(
            path='{base_api}/jobs/{pipeline_name}/{stage_name}/{job_name}/history/{offset}'.format(
                base_api=self.base_api,
                pipeline_name=pipeline_name,
                stage_name=stage_name,
                job_name=job_name,
                offset=offset
            ),
            headers={'Accept': 'application/xml'},
        )

        instances = list()
        for instance in response.json().get('jobs'):
            instances.append(JobInstance(session=self._session, data=instance, stage=None))

        return instances


class JobInstance(Base):
    """
    Class representing specific job.

    Instances of this class can get different information about the job:
        * get pipeline name/counter
        * get stage name/counter
        * list available artifacts
        * list available properties
    """

    def __init__(self, session, data, stage):
        super(JobInstance, self).__init__(session, data)
        self._stage = stage

    @property
    def pipeline_name(self):
        """
        Get pipeline name of current job instance.

        Because instantiating job instance could be performed in different ways and those return different results,
        we have to check where from to get name of the pipeline.

        :return: pipeline name.
        """
        if 'pipeline_name' in self.data:
            return self.data.get('pipeline_name')
        elif self.stage.pipeline is not None:
            return self.stage.pipeline.data.name
        else:
            return self.stage.data.pipeline_name

    @property
    def pipeline_counter(self):
        """
        Get pipeline counter of current job instance.

        Because instantiating job instance could be performed in different ways and those return different results,
        we have to check where from to get counter of the pipeline.

        :return: pipeline counter.
        """
        if 'pipeline_counter' in self.data:
            return self.data.get('pipeline_counter')
        elif self.stage.pipeline is not None:
            return self.stage.pipeline.data.counter
        else:
            return self.stage.data.pipeline_counter

    @property
    def stage_name(self):
        """
        Get stage name of current job instance.

        Because instantiating job instance could be performed in different ways and those return different results,
        we have to check where from to get name of the stage.

        :return: stage name.
        """
        if 'stage_name' in self.data:
            return self.data.get('stage_name')
        else:
            return self.stage.data.name

    @property
    def stage_counter(self):
        """
        Get stage counter of current job instance.

        Because instantiating job instance could be performed in different ways and those return different results,
        we have to check where from to get counter of the stage.

        :return: stage counter.
        """
        if 'stage_counter' in self.data:
            return self.data.get('stage_counter')
        else:
            return self.stage.data.counter

    @property
    def url(self):
        """
        Returns url for accessing job instance.
        """
        return (
            "{server_url}/go/tab/build/detail"
            "/{pipeline_name}/{pipeline_counter}/{stage_name}/{stage_counter}/{job_name}").format(
            server_url=self._session.server_url,
            pipeline_name=self.pipeline_name,
            pipeline_counter=self.pipeline_counter,
            stage_name=self.stage_name,
            stage_counter=self.stage_counter,
            job_name=self.data.name
        )

    @property
    def stage(self):
        return self._stage

    @property
    def artifact(self):
        """
        Property for accessing artifact manager of the current job.

        :return: instance of :class:`yagocd.resources.artifact.ArtifactManager`
        :rtype: yagocd.resources.artifact.ArtifactManager
        """
        return ArtifactManager(
            session=self._session,
            pipeline_name=self.pipeline_name,
            pipeline_counter=self.pipeline_counter,
            stage_name=self.stage_name,
            stage_counter=self.stage_counter,
            job_name=self.data.name
        )

    @property
    def prop(self):
        """
        Property for accessing property (doh!) manager of the current job.

        :return: instance of :class:`yagocd.resources.property.PropertyManager`
        :rtype: yagocd.resources.property.PropertyManager
        """
        return PropertyManager(
            session=self._session,
            pipeline_name=self.pipeline_name,
            pipeline_counter=self.pipeline_counter,
            stage_name=self.stage_name,
            stage_counter=self.stage_counter,
            job_name=self.data.name
        )


if __name__ == '__main__':
    pass



#!/usr/bin/env python
# -*- coding: utf-8 -*-

###############################################################################
# The MIT License
#
# Copyright (c) 2016 Grigory Chernyshev
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.
#
###############################################################################

from yagocd.resources import BaseManager


class FeedManager(BaseManager):
    """
    The feed API allows users to view feed information.
    """

    def pipelines(self):
        """
        Lists all pipelines.

        :return: an array of pipelines in XML format.
        """
        response = self._session.get(
            path='{base_api}/pipelines.xml'.format(base_api=self.base_api),
            headers={'Accept': 'application/xml'},
        )

        return response.text

    def pipeline_by_id(self, pipeline_id):
        """
        Gets XML representation of pipeline.

        :param pipeline_id: id of pipeline. Note: this is *not* a counter.
        :return: a pipeline object in XML format.
        """
        response = self._session.get(
            path='{base_api}/pipelines/{pipeline_name}/{pipeline_id}.xml'.format(
                base_api=self.base_api,
                pipeline_name='THIS_PARAMETER_IS_USELESS',  # WTF?!!
                pipeline_id=pipeline_id,
            ),
            headers={'Accept': 'application/xml'},
        )

        return response.text

    def stages(self, pipeline_name):
        """
        Gets feed of all stages for the specified pipeline with links to the pipeline and stage details.

        :param pipeline_name: name of pipeline, for which to list stages
        :return: an array of feed of stages in XML format.
        """
        response = self._session.get(
            path='{base_api}/pipelines/{pipeline_name}/stages.xml'.format(
                base_api=self.base_api,
                pipeline_name=pipeline_name
            ),
            headers={'Accept': 'application/xml'},
        )

        return response.text

    def stage_by_id(self, stage_id):
        """
        Gets XML representation of stage.

        :param stage_id: id of stage. Note: this is *not* a counter.
        :return: a stage object in XML format.
        """
        response = self._session.get(
            path='{base_api}/stages/{stage_id}.xml'.format(
                base_api=self.base_api,
                stage_id=stage_id
            ),
            headers={'Accept': 'application/xml'},
        )

        return response.text

    def stage(self, pipeline_name, pipeline_counter, stage_name, stage_counter):
        """
        Gets XML representation of stage.

        :param pipeline_name: name of pipeline.
        :param pipeline_counter: pipeline counter.
        :param stage_name: name of stage.
        :param stage_counter: stage counter.
        :return: a stage object in XML format.
        """
        response = self._session.get(
            path='{base_api}/pipelines/{pipeline_name}/{pipeline_counter}/{stage_name}/{stage_counter}.xml'.format(
                base_api=self._session.base_api(api_path=''),  # WTF?!!
                pipeline_name=pipeline_name,
                pipeline_counter=pipeline_counter,
                stage_name=stage_name,
                stage_counter=stage_counter,
            ),
            headers={'Accept': 'application/xml'},
        )

        return response.text

    def job_by_id(self, job_id):
        """
        Gets XML representation of job.

        :param job_id: id of job. Note: this is *not* a counter.
        :return: a job object in XML format.
        """
        response = self._session.get(
            path='{base_api}/jobs/{job_id}.xml'.format(
                base_api=self.base_api,
                job_id=job_id
            ),
            headers={'Accept': 'application/xml'},
        )

        return response.text


if __name__ == '__main__':
    pass
